
/******************************************************************************
 *
 *  Quest Software, Inc.
 * ALL RIGHTS RESERVED.
 *
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 * 
 *  *  Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.  
 * 
 *  *  Redistributions in binary form must reproduce the above copyright notice, 
 *     this list of conditions and the following disclaimer in the documentation 
 *     and/or other materials provided with the distribution.  
 * 
 *  *  Neither the name of Quest Software, Inc. nor the names of its contributors 
 *     may be used to endorse or promote products derived from this software 
 *     without specific prior written permission. 
 * 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ON AN "AS IS" 
 * AND UNSUPPORTED BASIS, WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES OF ANY KIND, 
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, NON-
 * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE COPYRIGHT 
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ************************************************************************************/


#include "loadConfigurationDialog.h"
#include <QtDebug>

LoadConfigurationDialog::LoadConfigurationDialog(Farm *farm, QWidget *parent /*= 0*/)
	: QDialog(parent)
{
	setupUi(this);

	this->farm = farm;

	// Network manager initialization
	manager = new QNetworkAccessManager(this);
	connect(manager, SIGNAL(finished(QNetworkReply*)),
		this, SLOT(replyFinished(QNetworkReply*)));

	// Hide browse button
	browseButton->setVisible(false);

	// Connecting signals with slots
	connect(protocolComboBox, SIGNAL(currentIndexChanged(int)),
		SLOT(currentProtocolChanged(int)));
	connect(browseButton, SIGNAL(clicked()), SLOT(browseFile()));
	connect(manuallyRadioButton, SIGNAL(toggled(bool)),
		SLOT(manuallyChecked(bool)));
	connect(okButton, SIGNAL(clicked()), SLOT(accept()));
        connect(cancelButton, SIGNAL(clicked()), SLOT(cancel()));

        this->loadSettings();
}

LoadConfigurationDialog::~LoadConfigurationDialog()
{

}

void LoadConfigurationDialog::currentProtocolChanged(int index)
{
	// If file then show browse button
	QString currentProtocol = protocolComboBox->itemText(index);
	bool browseButtonVisible;

	if (currentProtocol.compare("FILE", Qt::CaseInsensitive) == 0)
	{
		browseButtonVisible = true;
		urlLabel->setText(tr("Dir:"));
		proxyGroupBox->setEnabled(false);
	}
	else
	{
		browseButtonVisible = false;
		urlLabel->setText(tr("URL:"));
		proxyGroupBox->setEnabled(true);
	}
	browseButton->setVisible(browseButtonVisible);
}

void LoadConfigurationDialog::browseFile()
{
	// Getting file name from dialog
	QString absName = QFileDialog::getOpenFileName(this, tr("Open File"), "/home",
                                                     tr("XML files (*.xml)"));
	// Nothing actions if name = 0
	if (absName.isEmpty())
		return;

	// Parse name
	int index = absName.lastIndexOf('/');
	QString dir = absName.mid(0, index);
	QString name = absName.mid(index + 1, absName.length() - index);

	// Add dirname and filename to edits
	urlEdit->setText(dir);
	fileEdit->setText(name);
}

void LoadConfigurationDialog::manuallyChecked(bool checked)
{
	//If manually internet settings is checked
	proxyEdit->setEnabled(checked);

	// If manually isn`t checked then clear proxy edit
	if (!checked)
		proxyEdit->setText("");
}

void LoadConfigurationDialog::accept()
{
    // Loading farm
    bool result = this->loadFarm();
    if (!result)
    {
        QMessageBox::information(this, this->windowTitle(), tr("The following error occured while "
                                                  "loading the farm definition from [") + urlEdit->text() + "/" + fileEdit->text() +
                                                  tr("]: Invalid configuration file.\nPlease specify a valid file for farm definition."));
        return;
    }
    else
    {
        QMessageBox::information(this, this->windowTitle(), tr("Configuration [") + farm->getName() +
                                                  tr("] loaded successfully from [") + urlEdit->text() + "/" + fileEdit->text() + tr("]."));
    }


    // TODO: выход в одном месте

    // Saving farm settings to next settings
    this->saveSettings();
    return QDialog::accept();
}

void LoadConfigurationDialog::cancel()
{
        return QDialog::reject();
}

bool LoadConfigurationDialog::loadFarm()
{
    QString currentProtocol = protocolComboBox->currentText();

	if (urlEdit->text().isEmpty())
	{
	    if (currentProtocol.compare("FILE", Qt::CaseInsensitive) == 0)
	    {
            QMessageBox::critical(this, tr("Configuration error"),
                tr("Please, select file"));
	    }
	    else
	    {
	        QMessageBox::critical(this, tr("URL error"),
			tr("Enter correct URL"));
	    }
        return false;
	}

	if (fileEdit->text().isEmpty())
	{
		QMessageBox::critical(this, tr("File error"),
			tr("Enter correct file name"));
		return false;
	}

	if (currentProtocol.compare("FILE", Qt::CaseInsensitive) == 0)
	{
		// Loading from disk file
		QString absName = urlEdit->text() + "/" + fileEdit->text();

		// If file can not be read
		QFileInfo fileInfo(absName);
		if (!fileInfo.exists())
		{
			QMessageBox::critical(this, tr("File read error"),
				tr("File doesn`t exist!"));
         		return false;
		}

		// If we can to read this file
		QFile file(absName);
		QByteArray parseBuffer;
		if (file.open(QIODevice::ReadOnly))
		{
			parseBuffer = file.readAll();
			file.close();
		}
		else
		{
			QMessageBox::critical(this, tr("File read error"),
				tr("File couldn`t be read!"));
         		return false;
		}

		return farm->fromBuffer(parseBuffer);
	}
	else
	{
		// Getting host name
		QString host = urlEdit->text();
		QString name = fileEdit->text();
		QUrl url(currentProtocol.toLower() + "://" +
			host + "/" + name);

		// Getting proxy
		if (manuallyRadioButton->isChecked())
		{
			QString proxyList = proxyEdit->text();

			// Getting proxyserver and proxyport
			int index = proxyList.lastIndexOf(':');
			QString proxyserver = proxyList.mid(0, index);
			QString proxyport = proxyList.mid(index, proxyList.length() - index - 1);

			// Converting string port to int port
			bool okPortToInt;
			int port = proxyport.toInt(&okPortToInt);
			if (!okPortToInt)
			{
				QMessageBox::critical(this, tr("Connection error"),
					tr("Enter correct value of proxy port"));
         			return false;
			}

			// Setting proxy to connection
			QNetworkProxy proxy;
			proxy.setPort(port);
			proxy.setHostName(proxyserver);

			manager->setProxy(proxy);
		}

                manager->get(QNetworkRequest(url));
                return false;
	}
}

void LoadConfigurationDialog::replyFinished(QNetworkReply* reply)
{
	if (reply->error() == QNetworkReply::NoError)
	{
		// Reading data
		QByteArray parseBuffer = reply->readAll();

                //qDebug() << parseBuffer;

                // Parsing data
		if (!farm->fromBuffer(parseBuffer))
		{
			QMessageBox::critical(this, tr("Parsing error"),
				tr("Error in parsing data"));
		}
                this->saveSettings();
                return QDialog::accept();
	}
	else
	{
		QMessageBox::critical(this, tr("Connection error"),
                        tr("Error code: ") + reply->errorString());
	}
}

void LoadConfigurationDialog::saveSettings()
{
    farm->setSourceProtocol(protocolComboBox->currentText().toLower());
    farm->setSourceServer(urlEdit->text());
    farm->setSourceFile(fileEdit->text());

    farm->setSourceProxy(proxyEdit->text());
}

void LoadConfigurationDialog::loadSettings()
{
    int protocolIndex = protocolComboBox->findText(farm->getSourceProtocol(),
                                                   Qt::MatchFixedString);
    if (protocolIndex >= 0)
        protocolComboBox->setCurrentIndex(protocolIndex);

    urlEdit->setText(farm->getSourceServer());
    fileEdit->setText(farm->getSourceFile());

    QString proxy = farm->getSourceProxy();
    if (proxy.isEmpty())
        defaultRadioButton->setChecked(true);
    else
    {
        manuallyRadioButton->setChecked(true);
        proxyEdit->setText(proxy);
    }
}
