
/******************************************************************************
 *
 *  Quest Software, Inc.
 * ALL RIGHTS RESERVED.
 *
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 * 
 *  *  Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.  
 * 
 *  *  Redistributions in binary form must reproduce the above copyright notice, 
 *     this list of conditions and the following disclaimer in the documentation 
 *     and/or other materials provided with the distribution.  
 * 
 *  *  Neither the name of Quest Software, Inc. nor the names of its contributors 
 *     may be used to endorse or promote products derived from this software 
 *     without specific prior written permission. 
 * 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ON AN "AS IS" 
 * AND UNSUPPORTED BASIS, WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES OF ANY KIND, 
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, NON-
 * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE COPYRIGHT 
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ************************************************************************************/


#include "titlePage.h"
#include "LoadConfigurationDialog/loadConfigurationDialog.h"
#include <QtDebug>
#include <QInputDialog>

TitlePage::TitlePage(Farm *farm, ManageConnectionDialog *parent)
    : ManageConnectionPage(farm, parent)
{
	setupUi(this);

    FarmSettings *settings = parentDialog->getFarmSettings();

    farmList->addItems(settings->getFarmNamesList());

    if (farmList->count() != 0)
    {
        modifyExistingFarm->setChecked(true);
        if (farm != NULL)
        {
            farmList->setCurrentRow(parentDialog->getFarmSettings()->getFarmNamesList().indexOf(farm->getName()));
        }
        else
        {
            farmList->setCurrentRow(0);
        }
    }

    connect(createNewFarm, SIGNAL(toggled(bool)), SLOT(checkRadioButtonsState(bool)));
    connect(reloadButton, SIGNAL(clicked()), SLOT(loadFarm()));

    connect(farmList, SIGNAL(currentItemChanged(QListWidgetItem*, QListWidgetItem*)), SLOT(currentFarmChanged(QListWidgetItem*, QListWidgetItem*)));
    connect(deleteButton, SIGNAL(clicked()), SLOT(deleteFarm()));

    connect(renameButton, SIGNAL(clicked()), SLOT(renameCurrentFarm()));
    connect(autoConnect, SIGNAL(stateChanged(int)), SLOT(autoConnectedFarmChanged(int)));

    checkRadioButtonsState();
}

TitlePage::~TitlePage()
{

}

void TitlePage::renameCurrentFarm()
{
    bool ok = false;
    bool result = false;

    QString newName(currentFarm->getName());

    do
    {
        newName = QInputDialog::getText(this, parentDialog->windowTitle(), tr("Input new farm name:"), QLineEdit::Normal, newName, &ok);

        // Verifing new name
        result = currentFarm->verifyName(newName);

        if (!result && ok)
        {
            QMessageBox::information(this, tr("Error"), tr("Please, enter another farm name"));
        }
    }
    while (ok && !result);

    if (ok)
    {
        FarmSettings *settings = parentDialog->getFarmSettings();
        Farm *farm = settings->getFarm(currentFarm->getName());
        farm->setName(newName);
        currentFarm->setName(newName);
        parentDialog->loadFarmSettings();
        farmList->currentItem()->setText(currentFarm->getName());
    }
}

void TitlePage::currentFarmChanged(QListWidgetItem  *current, QListWidgetItem  *previous)
{
    if (current == NULL)
    {
        return;
    }

    parentDialog->setDialogFarm(false, current->text());
    parentDialog->loadFarmSettings();
}

void TitlePage::checkRadioButtonsState(bool isChecked)
{
    if (farmList->count() == 0)
    {
        modifyExistingFarm->setEnabled(false);
    }
    else
    {
        modifyExistingFarm->setEnabled(true);
    }

    if (createNewFarm->isChecked())
    {
        existingGroupBox->setEnabled(false);
        parentDialog->setDialogMode(true);
        parentDialog->setDialogFarm(true, QString());
        parentDialog->loadFarmSettings();
    }
    else
    {
        existingGroupBox->setEnabled(true);
        parentDialog->setDialogMode(false);
        parentDialog->setDialogFarm(false, farmList->currentItem()->text());
        parentDialog->loadFarmSettings();
    }
}

void TitlePage::loadFarm()
{
    // Create configuration dialog
    LoadConfigurationDialog *loadConfigurationDialog = new LoadConfigurationDialog(currentFarm, this);
    loadConfigurationDialog->setWindowTitle(tr("New Configuration"));

    // Show configuration dialog
    if (loadConfigurationDialog->exec() == QDialog::Accepted && currentFarm != NULL)
    {
        emit farmSettingsChanged();
        farmList->currentItem()->setText(currentFarm->getName());
    }
}

void TitlePage::deleteFarm()
{
    if (QMessageBox::question(this, this->windowTitle(),
                              tr("You are about to delete the settings for farm [") + (farmList->currentItem()->text()) + "].\n"
                              + tr("This cannot be undone.\n")
                              + tr("Are you sure you want to continue?"),
                              QMessageBox::Yes | QMessageBox::No) == QMessageBox::No)
    {
        return;
    }

    FarmSettings *settings = parentDialog->getFarmSettings();

    int farmIndex = farmList->currentRow();
    settings->removeFarm(farmIndex);

    farmList->clear();
    QStringList farmNames = settings->getFarmNamesList();

    // Enable work buttons
    bool isFarms = !farmNames.isEmpty();

    if (!isFarms)
    {
        createNewFarm->setChecked(true);
        return;
    }

    farmList->addItems(farmNames);
    modifyExistingFarm->setChecked(true);
    farmList->setCurrentRow(0);
}

void TitlePage::autoConnectedFarmChanged(int state)
{
    if (createNewFarm->isChecked())
        return;

    // set the AutoConnect option for the current farm and clear for all the others
    FarmSettings *settings = parentDialog->getFarmSettings();
    farmList->setFocus();
    QListWidgetItem *currentItem = farmList->currentItem();
    if (currentItem == 0)
        return;

    for(int i = 0; i < farmList->count(); ++i)
    {
        QListWidgetItem *item = farmList->item(i);
        QString name = item->text();
        Farm *farm = settings->getFarm(name);
        if (item == currentItem)
            farm->setAutoConnect(state != 0);
        else if (state != 0)
            farm->setAutoConnect(false);
    }
    currentFarm->setAutoConnect(state != 0);
}

bool TitlePage::nextEvents()
{
    bool ok = false;
    bool result = false;

    QString newName(currentFarm->getName());

    do
    {
        newName = QInputDialog::getText(this, parentDialog->windowTitle(), tr("Specify a name for the farm:"), QLineEdit::Normal, newName, &ok);

        // Verifing new name
        result = currentFarm->verifyName(newName);

        if (!result && ok)
        {
            QMessageBox::information(this, tr("Error"), tr("Please, enter another farm name"));
        }
    }
    while (ok && !result);

    if (ok)
    {
        currentFarm->setName(newName);
        parentDialog->loadFarmSettings();
    }

    return ok;
}

void TitlePage::loadFarmSettings()
{
    if (currentFarm->autoConnected())
    {
        autoConnect->setCheckState(Qt::Checked);
    }
    else
    {
        autoConnect->setCheckState(Qt::Unchecked);
    }
}
