
/******************************************************************************
 *
 *  Quest Software, Inc.
 * ALL RIGHTS RESERVED.
 *
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 * 
 *  *  Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.  
 * 
 *  *  Redistributions in binary form must reproduce the above copyright notice, 
 *     this list of conditions and the following disclaimer in the documentation 
 *     and/or other materials provided with the distribution.  
 * 
 *  *  Neither the name of Quest Software, Inc. nor the names of its contributors 
 *     may be used to endorse or promote products derived from this software 
 *     without specific prior written permission. 
 * 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ON AN "AS IS" 
 * AND UNSUPPORTED BASIS, WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES OF ANY KIND, 
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, NON-
 * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE COPYRIGHT 
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ************************************************************************************/


#include "farm.h"
#include "sublib/device.h"
#include "sublib/code.h"
#include "farmSettings.h"
#include <QDomDocument>
#include <QDomElement>
#include <QtDebug>

Farm::Farm(QSettings *s, QString key, const QString & name)
{
        this->settings = s;
        this->key = key;
        this->name = name;

        // No existing locations was found
        locations.clear();
        for (int i = 0; i < 3; i++)
        {
            Location *location = new Location();
            location->setName("Location " + QString::number(i + 1));
            locations << location;
        }

        kerberosMode = false;
        kerberosEnabled = false;
        ssoEnabled = false;
        saveCredentials = false;

        wallpaperEnabled = false;
        draggingEnabled = false;
        animationEnabled = false;
        themesEnabled  = false;
        bitmapCachingEnabled = false;
        reconnect = false;

        desktopWidth = 0;
        desktopHeight = 0;
        colorDepth = 0;
        connectionBar = false;
        pinConnectionBar = false;
        spanMonitors = false;
        smartSizing = false;
        seamlessMode = false;

        redirectClipboard = false;
        redirectDrives = false;
        redirectPrinters = false;
        redirectUPrinters = false;
        redirectComPorts = false;
        redirectHandhelds = false;
        redirectMicrophone = false;
        redirectSmartCards = false;

        downstreamAudioEnabled = false;
        daMode = "Local";
        daQuality = "cd";
        daBuffering = 0;
        daCustomQuality = 0;

        upstreamAudioEnabled = false;
        uaMode = "wave";
        uaQuality = 0;
        uaComplexity = 0;
        uaEnhancedCompression = false;

        hideSettings = false;
        pmPort = 443;
        allowPasswordManagement = true;

        iLast = 0;
        iDefault = 0;
        promptLocation = true;
        currentLocation = 0;

        needReconnect = false;
        autoConnect = false;
}

Farm::~Farm()
{
}

void Farm::setName(const QString & newName)
{
    name = newName;
}

bool Farm::verifyName(const QString & newName)
{
    if ( (newName.isEmpty()) || (newName.trimmed().isEmpty()) )
        return false;

    // Get parent settings
    FarmSettings *farmSettings = (FarmSettings *) settings;
    if (farmSettings == 0)
        return true;

    QStringList allNamesList = farmSettings->getFarmNamesList();
    foreach (QString name, allNamesList)
    {
        if (name == newName)
        {
            Farm *farm = farmSettings->getFarm(name);
            if (this->getKey() != farm->getKey())
                return false;
        }
    }

    return true;
}

QString Farm::getName() const
{
    return this->name;
}

QFile * Farm::createPitFile()
{
	QTemporaryFile *pitFile = new QTemporaryFile;

	if (!pitFile->open())
                return 0;

	// Write farm settings in pit-file
	QTextStream stream(pitFile);
	stream << "username:s:" << getUsername() << endl
            << "password:s:" << getPassword() << endl
            << "domain:s:" << getCurrentDomain() << endl;

        // Location settings
        if (currentLocation != 0)
        {
            stream << "llmservers:s:" << currentLocation->getServers().join(",") << endl
                << "llmtcpport:s:" << currentLocation->getPort() << endl
                << "llmenablenat:i:" << (int) currentLocation->isNat() << endl
                << "enablessl:i:" << (int) currentLocation->isSsl() << endl
                << "sslgateway:s:" << currentLocation->getSslGateway() << endl;

            int protocol;
            if (currentLocation->getProtocol() == "http")
                protocol = 0;
            else
		protocol = 1;
            stream << "llmprotocol:i:" << protocol << endl;
        }

        if (waitForDesktop())
        {
            stream << "wait:i:" << (int) getWaitTime() << endl
                << "retry:i:" << (int) getNumberOfRetries() << endl;
        }

        // User experience options
        stream << "disable wallpaper:i:" << (int) (isWallpaperEnabled() == false) << endl
            << "disable full window drag:i:" << (int) (isDraggingEnabled() == false) << endl
            << "disable menu anims:i:" << (int) (isAnimationEnabled() == false) << endl
            << "disable themes:i:" << (int) (isThemesEnabled() == false) << endl
            << "bitmapcachepersistenable:i:" << (int) isBitmapCachingEnabled() << endl;

	int width = getDesktopWidth();
	int height = getDesktopHeight();
	QString desktopSize;
	if (width && height)
		desktopSize = QString::number(width) + "x" + QString::number(height);
	else
                desktopSize = "fullscreen";

        // Display and sound options
        stream << "windowSize:s:" << desktopSize << endl
            << "session bpp:s:" << getColorDepth() << endl
            << "sound:s:" << getSoundMode() << endl;

        // Seamless mode
        stream << "seamlessMode:i:" << (int) isSeamlessMode() << endl;

        // Multi-monitor feature
        stream << "spanmonitors:i:" << (int) isSpanMonitors() << endl;


	// Redirection options
        DeviceDirector deviceDirector;
	if (this->isRedirectDrives())
	{
		QStringList drives = deviceDirector.getMountDrives();
		stream << "drives:s:" << drives.join(",") << endl;
	}

        if (this->isRedirectComPorts())
	{
		QStringList comPorts = deviceDirector.getComPorts();
		stream << "comports:s:" << comPorts.join(",") << endl;
	}

	if (this->isRedirectPrinters())
	{
		QString printerName = deviceDirector.getPrinterName();
		if (!printerName.isEmpty())
		{
			stream << "printers:s:" << printerName << endl;
		}
	}
        stream << "downstreamAudioEnabled:i:" << (int) this->isDownstreamAudioEnabled() << endl;
        stream << "daMode:s:" << this->getDAMode() << endl;
        stream << "daQuality:s:" << this->getDAQuality() << endl;
        stream << "daBuffering:s:" << (int) this->getDABuffering() << endl;
        stream << "daCustomQuality:s:" << (int) this->getDACustomQuality() << endl;

        stream << "upstreamAudioEnabled:i:" << (int) this->isUpstreamAudioEnabled() << endl;
        stream << "uaMode:s:" << this->getUAMode() << endl;
        stream << "uaQuality:s:" << (int) this->getUAQuality() << endl;
        stream << "uaComplexity:s:" << (int) this->getUAComplexity() << endl;
        stream << "uaEnhancedCompression:i:" << (int) this->getUAEnhancedCompression() << endl;

        // Other redirections
        stream << "redirectclipboard:i:" << (int) this->isRedirectClipboard() << endl;

        // USB redirection handhelds
        stream << "redirecthandhelds:i:" << (int) this->isRedirectHandhelds() << endl;

        // MMR Redirect
        stream << "multimediaredirection:i:" << (int) this->isMmrEnabled() << endl;

        // FMMR Redirect
        stream << "flashredirection:i:" << (int) this->isFmmrEnabled() << endl;

        // GA Enable
        stream << "bitmapacceleration:i:" << (int) this->isGaEnabled() << endl;

        pitFile->close();
	return pitFile;
}

void Farm::saveFarm()
{
	settings->beginGroup(key);

	settings->setValue("Name", name);
	if (saveCredentials)
	{
	    settings->setValue("Username", username);
	    settings->setValue("Password",
                           encryptString256(password));
	    settings->setValue("Domain", currentDomain);
	}
	domainList.removeAll("");
	settings->setValue("DomainList", domainList);
	settings->setValue("AllowSaveCredentials", saveCredentials);
        settings->setValue("KerberosMode", kerberosMode);
        settings->setValue("KerberosEnabled", kerberosEnabled);
        settings->setValue("SSOEnabled", ssoEnabled);

        settings->setValue("DefaultLocation", iDefault);
        settings->setValue("PromptForLocation", promptLocation);
        //settings->setValue("LastLocation", iLast);

    settings->setValue("WaitTime", waitTime);
    settings->setValue("NumberOfRetries", numberOfRetries);
    settings->setValue("WaitForDesktopEnabled", waitForDesktopIsEnabled);

	settings->setValue("EnableWallpaper", wallpaperEnabled);
	settings->setValue("EnableFullWindowDrag", draggingEnabled);
	settings->setValue("EnableAnimation", animationEnabled);
	settings->setValue("EnableThemes", themesEnabled);
        settings->setValue("EnableBitmapCaching", bitmapCachingEnabled);
        settings->setValue("mmr", mmrEnabled);
        settings->setValue("fmmr", fmmrEnabled);
        settings->setValue("graphicsAccel", gaEnabled);
        settings->setValue("Reconnect", reconnect);

	settings->setValue("RedirectClipboard", redirectClipboard);
	settings->setValue("RedirectDrives", redirectDrives);
	settings->setValue("RedirectPrinters", redirectPrinters);
	settings->setValue("RedirectUniversalPrinters", redirectUPrinters);
	settings->setValue("RedirecComPorts", redirectComPorts);
	settings->setValue("RedirectHandhelds", redirectHandhelds);
	settings->setValue("RedirectMicrophone", redirectMicrophone);
	settings->setValue("RedirectSmartCards", redirectSmartCards);

        settings->setValue("DisplayConnectionBar", connectionBar);
        settings->setValue("PinConnectionBar", pinConnectionBar);
        settings->setValue("SpanMonitors", spanMonitors);
        settings->setValue("SmartSizing", smartSizing);
        settings->setValue("SeamlessMode", seamlessMode);
        settings->setValue("RedirectAudio", soundMode);

    settings->setValue("EnableDownstreamAudio", downstreamAudioEnabled);
    settings->setValue("DAMode", daMode);
    settings->setValue("DAQuality", daQuality);
    settings->setValue("DABuffering", daBuffering);
    settings->setValue("DACustomQuality", daCustomQuality);

    settings->setValue("EnableUpstreamAudio", upstreamAudioEnabled);
    settings->setValue("UAMode", uaMode);
    settings->setValue("UAQuality", uaQuality);
    settings->setValue("UAComplexity", uaComplexity);
    settings->setValue("UAEnhancedCompression", uaEnhancedCompression);

	settings->setValue("DesktopWidth", desktopWidth);
	settings->setValue("DesktopHeight", desktopHeight);
	settings->setValue("ColorDepth", colorDepth);
	settings->setValue("HideSettings", hideSettings);

        settings->setValue("AllowPasswordManagement", allowPasswordManagement);
        settings->setValue("PasswordManagementPort", pmPort);
        settings->setValue("PasswordManagementServer", pmServer);

        settings->setValue("SourceServer", sourceServer);
        settings->setValue("SourceProtocol", sourceProtocol);
        settings->setValue("SourceFile", sourceFile);
        settings->setValue("SourceProxy", sourceProxy);

        settings->setValue("AutoLaunch", p_autolaunch.cache());
        settings->setValue("Applications", p_applications.cache());

	for (int i = 0; i < locations.count(); i++)
	{
		settings->beginGroup("Location" + QString::number(i + 1));

		settings->setValue("Name", locations.at(i)->getName());
		settings->setValue("NAT", locations.at(i)->isNat());
		settings->setValue("RDPonSSL", locations.at(i)->isSsl());
		settings->setValue("SSLGateway", locations.at(i)->getSslGateway());
		settings->setValue("Servers", locations.at(i)->getServers());
		settings->setValue("TCPPort", locations.at(i)->getPort());
                settings->setValue("Protocol", locations.at(i)->getProtocol());
                settings->setValue("ProxyBypassList", locations.at(i)->getProxyBypassList());
                settings->setValue("ProxyServer", locations.at(i)->getProxyServer());

		settings->endGroup();
	}

	settings->setValue("AutoConnect", autoConnect);

	settings->endGroup();
}

bool Farm::loadFarm()
{
	settings->beginGroup(key);

	name = settings->value("Name").toString();
	username = settings->value("Username").toString();
        password = decryptString256(settings->value("Password").toString());
	domainList = settings->value("DomainList").toStringList();
	domainList = QStringList::fromSet(domainList.toSet());
	currentDomain = settings->value("Domain").toString();
	saveCredentials = settings->value("AllowSaveCredentials").toBool();
        kerberosMode = settings->value("KerberosMode").toBool();
        kerberosEnabled = settings->value("KerberosEnabled").toBool();
        ssoEnabled = settings->value("SSOEnabled").toBool();

        iDefault = settings->value("DefaultLocation").toInt();
        promptLocation = settings->value("PromptForLocation", true).toBool();
        //iLast = settings->value("LastLocation").toInt();

    waitForDesktopIsEnabled = settings->value("WaitForDesktopEnabled", true). toBool();
    waitTime = settings->value("WaitTime", 6).toInt();
    numberOfRetries = settings->value("NumberOfRetries", 20).toInt();

	wallpaperEnabled = settings->value("EnableWallpaper").toBool();
	draggingEnabled = settings->value("EnableFullWindowDrag").toBool();
	animationEnabled = settings->value("EnableAnimation").toBool();
	themesEnabled = settings->value("EnableThemes").toBool();
        bitmapCachingEnabled = settings->value("EnableBitmapCaching").toBool();
        mmrEnabled = settings->value("mmr").toBool();
        fmmrEnabled = settings->value("fmmr").toBool();
        gaEnabled = settings->value("graphicsAccel").toBool();
        reconnect = settings->value("Reconnect").toBool();

        connectionBar = settings->value("DisplayConnectionBar").toBool();
        pinConnectionBar = settings->value("PinConnectionBar").toBool();
        spanMonitors = settings->value("SpanMonitors").toBool();
        smartSizing = settings->value("SmartSizing").toBool();
        seamlessMode = settings->value("SeamlessMode").toBool();
        soundMode = settings->value("RedirectAudio").toString();

	redirectClipboard = settings->value("RedirectClipboard").toBool();
	redirectDrives = settings->value("RedirectDrives").toBool();
	redirectPrinters = settings->value("RedirectPrinters").toBool();
	redirectUPrinters = settings->value("RedirectUniversalPrinters").toBool();
	redirectComPorts = settings->value("RedirecComPorts").toBool();
	redirectHandhelds = settings->value("RedirectHandhelds").toBool();
	redirectMicrophone = settings->value("RedirectMicrophone").toBool();
	redirectSmartCards = settings->value("RedirectSmartCards").toBool();

	downstreamAudioEnabled = settings->value("EnableDownstreamAudio").toBool();
	daMode = settings->value("DAMode").toString();
	daQuality = settings->value("DAQuality").toString();
	daBuffering = settings->value("DABuffering").toInt();
	daCustomQuality = settings->value("DACustomQuality").toInt();

	upstreamAudioEnabled = settings->value("EnableUpstreamAudio").toBool();
	uaMode = settings->value("UAMode").toString();
	uaQuality = settings->value("UAQuality").toInt();
	uaComplexity = settings->value("UAComplexity").toInt();
	uaEnhancedCompression = settings->value("UAEnhancedCompression").toBool();

	desktopWidth = settings->value("DesktopWidth").toInt();
	desktopHeight = settings->value("DesktopHeight").toInt();
	colorDepth = settings->value("ColorDepth").toInt();
        hideSettings = settings->value("HideSettings").toBool();

        allowPasswordManagement = settings->value("AllowPasswordManagement", true).toBool();
        pmPort = settings->value("PasswordManagementPort", "444").toInt();
        pmServer = settings->value("PasswordManagementServer").toString();

        sourceServer = settings->value("SourceServer").toString();
        sourceProtocol = settings->value("SourceProtocol").toString();
        sourceFile = settings->value("SourceFile").toString();
        sourceProxy = settings->value("SourceProxy").toString();

        //autoLaunch = settings->value("AutoLaunch").toStringList();
        QStringList autoLaunchCache = settings->value("AutoLaunch").toStringList();
        p_autolaunch = ApplicationList(autoLaunchCache);
        QStringList applicationsCache = settings->value("Applications").toStringList();
        p_applications = ApplicationList(applicationsCache);

        // Loading existing locations
	QStringList locationsList = settings->childGroups();
        locations.clear();

        for (int i = 0; i < locationsList.count(); i++)
	{
		settings->beginGroup(locationsList.at(i));

		Location *location = new Location();

		location->setName(settings->value("Name").toString());
		location->setNat(settings->value("NAT").toBool());
		location->setSsl(settings->value("RDPonSSL").toBool());
		location->setSslGateway(settings->value("SSLGateway").toString());
		location->setServers(settings->value("Servers").toStringList());
		location->setPort(settings->value("TCPPort", "443").toString());
                location->setProtocol(settings->value("Protocol", "http").toString());
                location->setProxyServer(settings->value("ProxyServer").toString());
                location->setProxyBypassList(settings->value("ProxyBypassList").toString());

		locations << location;

		settings->endGroup();
	}

	autoConnect = settings->value("AutoConnect").toBool();

        this->currentLocation = 0;
        this->setLastLocation(0);

	settings->endGroup();

	return true;
}

QStringList Farm::getLocationNamesList()
{
	QStringList names;
	for (int i = 0; i < locations.count(); i++)
		names << locations.at(i)->getName();
	return names;
}

bool Farm::fromBuffer(const QByteArray & buffer)
{
	QDomDocument xmlDoc;

	if (xmlDoc.setContent(buffer))
	{
		QDomElement firstElement = xmlDoc.documentElement();
                this->parseNextNode(firstElement);
                this->setLastLocation(0);
		return true;
	}

	return false;
}

void Farm::parseNextNode(const QDomNode & node)
{
	QDomNode domNode = node.firstChild();

	while (!domNode.isNull())
	{
		if (domNode.isElement())
		{
			QDomElement domElement = domNode.toElement();

			if (!domElement.isNull())
			{
				// If node is farm
				if (domElement.tagName() == "Farm")
				{
					// Farm name
					QString name = domElement.attribute("FarmName");
					if (!name.isEmpty())
						this->setName(name);
//					qDebug() << name;

					// Password management
					QString pmServer = domElement.attribute("PasswordManagementServer");
					this->setPasswordManagementServer(pmServer);
					QString spmPort = domElement.attribute("PasswordManagementPort", "443");
					int ipmPort = spmPort.toInt();
					this->setPasswordManagementPort(ipmPort);
					bool pmAllow = domElement.attribute("AllowPasswordManagement", "0").toInt();
					this->setPasswordManagementAllow(pmAllow);

                                        //domElement.attribute("DIShortcutLocations");

                                        bool connectionBar = domElement.attribute("DisplayConnectionBar", "0").toInt();
                                        this->setConnectionBar(connectionBar);
                                        bool pinConnectionBar = domElement.attribute("PinConnectionBar", "0").toInt();
                                        this->setPinConnectionBar(pinConnectionBar);
                                        bool spanMonitors = domElement.attribute("FullScreen", "0").toInt();
                                        this->setSpanMonitors(spanMonitors);
                                        bool smartSizing = domElement.attribute("EnableSmartSizing", "0").toInt();
                                        this->setSmartSizing(smartSizing);
                                        bool seamlessMode = domElement.attribute("SeamlessMode", "0").toInt();
                                        this->setSeamlessMode(seamlessMode);
					bool hideSettings = domElement.attribute("HideSettings", "0").toInt();
                                        this->setHideSettings(hideSettings);

                                        // Credentials options
                                        bool ssoEnabled = domElement.attribute("EnableSSO", "0").toInt();
                                        this->setSsoEnable(ssoEnabled);
                                        bool kerberos = domElement.attribute("EnableKerberos", "0").toInt();
                                        this->setKerberosEnable(kerberos);
                                        bool kerberosMode = domElement.attribute("KerberosMode", "0").toInt();
                                        this->setKerberosMode(kerberosMode);
                                        bool allowSave = !domElement.attribute("DisallowSaveCredentials", "0").toInt();
                                        this->setCredentialsSave(allowSave);

                                        // Location options
                                        bool promptLocation = domElement.attribute("PromptForLocation").toInt();
                                        this->setPromptLocation(promptLocation);
                                        int defaultLocation = domElement.attribute("DefaultLocation").toInt();
                                        this->setDefaultLocation(defaultLocation);

                                        int desktopWidth = domElement.attribute("DesktopWidth", "0").toInt();
                                        this->setDesktopWidth(desktopWidth);
                                        int desktopHeight = domElement.attribute("DesktopHeight", "0").toInt();
                                        this->setDesktopHeight(desktopHeight);
                                        int colorDepth = domElement.attribute("ColorDepth", "0").toInt();
                                        this->setColorDepth(colorDepth);

                                        // Sound mode switching
                                        int sound = domElement.attribute("AudioMode", "0").toInt();
                                        QString soundMode;
                                        switch (sound)
                                        {
                                        case 0:
                                            soundMode = "local";
                                            break;
                                        case 1:
                                            soundMode = "remote";
                                            break;
                                        default:
                                            soundMode = "off";
                                        }
                                        this->setSoundMode(soundMode);
                                        //domElement.attribute("KeyboardHook");

					// Redirection options
					QString rDrives = domElement.attribute("RedirectDrives", "0");
					this->setRedirectDrives(rDrives.toInt());
					QString rPrinters = domElement.attribute("RedirectPrinters", "0");
					this->setRedirectPrinters(rPrinters.toInt());
					QString rUPrinters = domElement.attribute("RedirectUniversalPrinters", "0");
					this->setRedirectUPrinters(rUPrinters.toInt());
					QString rCom = domElement.attribute("RedirectComPorts", "0");
					this->setRedirectComPorts(rCom.toInt());
					QString rCards = domElement.attribute("RedirectSmartCards", "0");
					this->setRedirectSmartCards(rCards.toInt());
					QString rHandhelds = domElement.attribute("RedirectHandhelds", "0");
					this->setRedirectHandhelds(rHandhelds.toInt());
					QString rClipboard = domElement.attribute("RedirectClipboard", "0");
					this->setRedirectClipboard(rClipboard.toInt());
					QString rMicrophone = domElement.attribute("RedirectMicrophone", "0");
					this->setRedirectMicrophone(rMicrophone.toInt());

					// User experience
					QString strWallpaper = domElement.attribute("EnableWallpaper", "0");
					this->setWallpaperEnable(strWallpaper.toInt());
					QString strWindowDrag = domElement.attribute("EnableFullWindowDrag", "0");
					this->setDraggingEnable(strWindowDrag.toInt());
					QString strAnimation = domElement.attribute("EnableAnimation", "0");
					this->setAnimationEnable(strAnimation.toInt());
					QString strThemes = domElement.attribute("EnableThemes", "0");
					this->setThemesEnable(strThemes.toInt());
					QString strBitmapCaching = domElement.attribute("EnableBitmapCaching", "0");
                                        this->setBitmapCachingEnable(strBitmapCaching.toInt());
                                        QString reconnect = domElement.attribute("AutoReconnect", "0");
                                        this->setReconnection(reconnect.toInt());
				}

                                if (!locations.isEmpty())
                                        locations.clear();
                                if (domElement.tagName() == "Location")
				{
                                    Location *location = new Location();

                                    // Number of reading location
                                    //int number = domElement.attribute("Number", "1").toInt() - 1;

                                    //if (number < 0 || number >= locations.count())
                                    //	continue;

                                    QString name = domElement.attribute("Name");
                                    if (!name.isEmpty())
                                        location->setName(name);

                                    // Setting the protocol port server
                                    int iProtocol = domElement.attribute("Protocol", "1").toInt();
                                    QString strProtocol = "https";
                                    if (iProtocol == 0)
                                        strProtocol = "http";
                                    location->setProtocol(strProtocol);

                                    QString port = domElement.attribute("TCPPort", "443");
                                    location->setPort(port);

                                    QString servers = domElement.attribute("ServerList");
                                    location->setServers(servers.split(","));

                                    // Setting ssl and nat
                                    bool ssl = domElement.attribute("RDPonSSL", "0").toInt();
                                    location->setSsl(ssl);
                                    bool nat = domElement.attribute("EnableNAT", "0").toInt();
                                    location->setNat(nat);
                                    QString gateway = domElement.attribute("SSLGateway");
                                    location->setSslGateway(gateway);

                                    // Proxy options
                                    QString proxyServer = domElement.attribute("ProxyServer");
                                    location->setProxyServer(proxyServer);
                                    QString proxyBypassList = domElement.attribute("ProxyBypassList");
                                    location->setProxyBypassList(proxyBypassList);

                                    locations << location;
				}
			}
		}
		this->parseNextNode(domNode);
		domNode = domNode.nextSibling();
	}
}

Location * Farm::createNewLocation()
{
    // Create new location name
    int newNumber = locations.count() + 1;
    QString newName = "Location " + QString::number(newNumber);

    // Initialize new location
    Location *newLocation = new Location();
    newLocation->setName(newName);

    return newLocation;
}

int Farm::addLocation(Location *location)
{
    // Appending location-parameter
    locations.append(location);

    return locations.indexOf(location);
}

void Farm::removeLocation(int number)
{
    locations.removeAt(number);
}

ApplicationList Farm::applications() const
{
    return p_applications;
}

void Farm::setApplications(ApplicationList apps)
{
    // TODO: clear cache when refreshing app list
    p_applications.clearWithCache();

    p_applications = apps;
}

ApplicationList Farm::autolaunch() const
{
    return p_autolaunch;
}

void Farm::setAutolaunch(ApplicationList apps)
{
    p_autolaunch.clear();

    p_autolaunch = apps;
}
