#!/bin/sh
#
# IncentivePro USB Server installation script
#

# Target directories
ROOTDIR=/usr/local                 # root dir where USB Server will be installed
PROGDIR=usb-server                 # USB Server directory name
BINDIR=bin                         # directory where binary files will be placed inside USB Server dir
MODULESBINDIR=modules/bin          # directory name where compiled stub driver modules are located
MODULESSRCDIR=modules/src          # directory name where stub driver sources are located
SHELLLINKDIR=/usr/local/bin        # directory to place symbolic link for USB Server Shell

# File names
STUBNAME=usbd                     # name of the stub driver
DAEMONNAME=usbsrvd                 # name of the daemon executable
SHELLNAME=usbsrv                   # name of the shell executable
INSTALLERNAME=installer.sh         # installer script name
STARTUPSCRIPTNAME=rc.usbsrvd       # daemon startup script name

# Startup script paths and symbolic link names
STARTUPSCRIPTDIR=/etc/init.d  # directory where daemon startup script will be placed
STARTUPDIR1=/etc/rc2.d
STARTUPDIR2=/etc/rc3.d
STARTUPDIR3=/etc/rc4.d
STARTUPDIR4=/etc/rc5.d
SHUTDOWNDIR1=/etc/rc6.d
SHUTDOWNDIR2=/etc/rc0.d
STARTUPLINK=S99$DAEMONNAME
SHUTDOWNLINK=K99$DAEMONNAME
# Alternate startup script paths for some other systems
ALTSTARTUPSCRIPTDIR=/etc/rc.d/init.d  # directory where daemon startup script will be placed
ALTSTARTUPDIR1=/etc/rc.d/rc2.d
ALTSTARTUPDIR2=/etc/rc.d/rc3.d
ALTSTARTUPDIR3=/etc/rc.d/rc4.d
ALTSTARTUPDIR4=/etc/rc.d/rc5.d
ALTSHUTDOWNDIR1=/etc/rc.d/rc6.d
ALTSHUTDOWNDIR2=/etc/rc.d/rc0.d
# One more alternate
ALTALTSTARTUPDIR=/etc/rcS.d


###############################################################################
#
# Helper subroutines

break_on_file_exist()
{
  if ( [ -h $1 ] || [ -f $1 ] || [ -x $1 ] ); then
    echo "!!!  Previous version installation detected. Please uninstall it first ($1)"
    exit 1
  fi
}

break_on_file_not_exist()
{
  if ( [ ! -f $1 ] && [ ! -x $1 ] && [ ! -h $1 ] ); then
    echo "!!!  Required file $1 is missing. Installer cannot continue."
    exit 2
  fi
}

break_on_wrong_kernel()
{
  if ( [ ! -d ./$MODULESBINDIR/$1 ] ); then
    echo "***  No stub driver for your kernel version. Please contact Quest Software (Provision Networks)."
    exit 3
  fi
}

break_on_error() {
  if [ $? -ne 0 ]; then
    echo "??? $1"
    usbsrv_uninstall
    exit 4
  fi
}

###############################################################################
#
# Installer and uninstaller subroutines

usbsrv_install()
{
  echo " "
  echo "*** Installing Quest Software (Provision Networks) USB Server..."

  # Remember the current working dir
  SCRIPTDIR=$PWD

  # Get the kernel version
  KERNELNAME=`uname -r`

  # Make sure we have a stub driver for this kernel version

  # Make sure no components from previous installation are installed
  
  break_on_file_exist $ROOTDIR/$PROGDIR/$BINDIR/$DAEMONNAME
  break_on_file_exist $ROOTDIR/$PROGDIR/$BINDIR/$STUBNAME.ko
  break_on_file_exist $ROOTDIR/$PROGDIR/$BINDIR/$SHELLNAME
  break_on_file_exist $ROOTDIR/$PROGDIR/$INSTALLERNAME
  break_on_file_exist $STARTUPSCRIPTDIR/$STARTUPSCRIPTNAME
  break_on_file_exist $STARTUPDIR1/$STARTUPLINK
  break_on_file_exist $STARTUPDIR2/$STARTUPLINK
  break_on_file_exist $STARTUPDIR3/$STARTUPLINK
  break_on_file_exist $STARTUPDIR4/$STARTUPLINK
  break_on_file_exist $SHUTDOWNDIR1/$SHUTDOWNLINK
  break_on_file_exist $SHUTDOWNDIR2/$SHUTDOWNLINK
  break_on_file_exist $SHELLLINKDIR/$SHELLNAME

  # Make sure we have all required files in the current dir
  break_on_file_not_exist ./$SHELLNAME
#  break_on_file_not_exist ./$MODULESBINDIR/$KERNELNAME/$STUBNAME.ko
  break_on_file_not_exist ./$DAEMONNAME
  break_on_file_not_exist ./$INSTALLERNAME
  break_on_file_not_exist ./$STARTUPSCRIPTNAME

  # Check if we have a compiled driver for this kernel
  # if we don't, then try to compile it from sources
  if ( [ ! -d ./$MODULESBINDIR/$KERNELNAME ] || [ ! -f ./$MODULESBINDIR/$KERNELNAME/$STUBNAME.ko ] ); then
    echo "***  No kernel module found for your kernel version. Will try to compile from sources."
    echo "***  Compiling kernel module..."

    cd ./$MODULESSRCDIR/$STUBNAME

    make >buildlog.txt 2>&1

    if ( [ $? -eq 0 ] ); then
      echo "***  Kernel module succesfully compiled"
      # Copy compiled module to binaries dir
      mkdir -p $SCRIPTDIR/$MODULESBINDIR/$KERNELNAME
      cp $STUBNAME.ko $SCRIPTDIR/$MODULESBINDIR/$KERNELNAME
      # Cleanup
      make clean > /dev/null
      cd $SCRIPTDIR
    else
      echo "***  Cannot compile kernel module. Installation terminated. See buildlog.txt file for more information."
      # Cleanup
      make clean > /dev/null
      cd $SCRIPTDIR
      exit 5
    fi
  fi

  # Check if the kernel module exists(whether precompiled or compiled from sources above) for this kernel version
  break_on_wrong_kernel $KERNELNAME

  echo "***  Copying files..."

  # Create program directory
  mkdir -p $ROOTDIR/$PROGDIR/$BINDIR
  break_on_error "Cannot create directory $ROOTDIR/$PROGDIR/$BINDIR"
  
  # Copy files
  cp $DAEMONNAME $ROOTDIR/$PROGDIR/$BINDIR
  break_on_error "Cannot copy file $DAEMONNAME"
  cp $MODULESBINDIR/$KERNELNAME/$STUBNAME.ko $ROOTDIR/$PROGDIR/$BINDIR
  break_on_error "Cannot copy file $STUBNAME.ko"
  cp $SHELLNAME $ROOTDIR/$PROGDIR/$BINDIR
  break_on_error "Cannot copy file $SHELLNAME"
  cp $INSTALLERNAME $ROOTDIR/$PROGDIR
  break_on_error "Cannot copy file $INSTALLERNAME"

  # Set permissions on copied files
  chmod 744 $ROOTDIR/$PROGDIR/$BINDIR/$DAEMONNAME
  break_on_error "Cannot chmod the daemon"
  chmod 644 $ROOTDIR/$PROGDIR/$BINDIR/$STUBNAME.ko
  break_on_error "Cannot chmod the driver"
  chmod 755 $ROOTDIR/$PROGDIR/$BINDIR/$SHELLNAME
  break_on_error "Cannot chmod the shell"
  chmod 744 $ROOTDIR/$PROGDIR/$INSTALLERNAME
  break_on_error "Cannot chmod the installer script"

  # Get the proper startup script dirs
  if ( [ ! -d $STARTUPSCRIPTDIR ] ); then
    echo "Using alt settings"
    STARTUPSCRIPTDIR=$ALTSTARTUPSCRIPTDIR
    STARTUPDIR1=$ALTSTARTUPDIR1
    STARTUPDIR2=$ALTSTARTUPDIR2
    STARTUPDIR3=$ALTSTARTUPDIR3
    STARTUPDIR4=$ALTSTARTUPDIR4
    SHUTDOWNDIR1=$ALTSHUTDOWNDIR1
	SHUTDOWNDIR2=$ALTSHUTDOWNDIR2
  fi

  if ( [ ! -d $STARTUPDIR ] ); then
    echo "Using alt-alt settings"
    STARTUPDIR=$ALTALTSTARTUPDIR
  fi

  # Convert '\' to '\/'
  ROOTDIRSTR=`echo $ROOTDIR | sed 's/\//\\\\\//g'`
  # Copy daemon startup script and setup program paths in it
  sed 's/%ROOTDIRTAG%/'$ROOTDIRSTR'/' $STARTUPSCRIPTNAME | sed 's/%PROGDIRTAG%/'$PROGDIR'/' | sed 's/%BINDIRTAG%/'$BINDIR'/' | sed 's/%STUBNAMETAG%/'$STUBNAME.ko'/' | sed 's/%DAEMONNAMETAG%/'$DAEMONNAME'/' > $STARTUPSCRIPTDIR/$STARTUPSCRIPTNAME
  break_on_error "Cannot copy file $STARTUPSCRIPTNAME"

  # Make the startup script executable
  chmod 744 $STARTUPSCRIPTDIR/$STARTUPSCRIPTNAME
  break_on_error "Cannot chmod the daemon startup script"

  # Create symbolic links
  ln -s $STARTUPSCRIPTDIR/$STARTUPSCRIPTNAME $STARTUPDIR1/$STARTUPLINK
  break_on_error "Cannot create link $STARTUPDIR/$STARTUPLINK in $STARTUPDIR1"
  ln -s $STARTUPSCRIPTDIR/$STARTUPSCRIPTNAME $STARTUPDIR2/$STARTUPLINK
  break_on_error "Cannot create link $STARTUPDIR/$STARTUPLINK in $STARTUPDIR2"
  ln -s $STARTUPSCRIPTDIR/$STARTUPSCRIPTNAME $STARTUPDIR3/$STARTUPLINK
  break_on_error "Cannot create link $STARTUPDIR/$STARTUPLINK in $STARTUPDIR3"
  ln -s $STARTUPSCRIPTDIR/$STARTUPSCRIPTNAME $STARTUPDIR4/$STARTUPLINK
  break_on_error "Cannot create link $STARTUPDIR/$STARTUPLINK in $STARTUPDIR4"
  ln -s $STARTUPSCRIPTDIR/$STARTUPSCRIPTNAME $SHUTDOWNDIR1/$SHUTDOWNLINK
  break_on_error "Cannot create link $SHUTDOWNDIR1/$SHUTDOWNLINK"
  ln -s $STARTUPSCRIPTDIR/$STARTUPSCRIPTNAME $SHUTDOWNDIR2/$SHUTDOWNLINK
  break_on_error "Cannot create link $SHUTDOWNDIR2/$SHUTDOWNLINK"
  ln -s $ROOTDIR/$PROGDIR/$BINDIR/$SHELLNAME $SHELLLINKDIR/$SHELLNAME
  break_on_error "Cannot create link $SHELLLINKDIR/$SHELLNAME"

  echo "***  Starting daemon..."

  # Start the daemon
  $STARTUPSCRIPTDIR/$STARTUPSCRIPTNAME start > /dev/null
  break_on_error "Cannot start USB Server"

  echo "***  Installation successful!"
  echo "***  Please allow incoming connections on 32032 port for USB Server to be able to accept connections from remote clients."
}

usbsrv_uninstall()
{
  echo " "
  echo "*** Uninstalling Quest Software (Provision Networks) USB Server..."

  echo "***  Stopping daemon..."

  killall -q -1 $DAEMONNAME

  # Wait while daemon terminates
  while ( [ -n "`ps -A|grep $DAEMONNAME`" ] ); do
    sleep 1
  done
  
  /sbin/rmmod $STUBNAME.ko 2> /dev/null

  echo "***  Removing files..."

  # Remove program files and symbolic links
  rm -f $ROOTDIR/$PROGDIR/$BINDIR/$DAEMONNAME
  rm -f $ROOTDIR/$PROGDIR/$BINDIR/$STUBNAME.ko
  rm -f $ROOTDIR/$PROGDIR/$BINDIR/$SHELLNAME
  rm -f $STARTUPDIR1/$STARTUPLINK
  rm -f $STARTUPDIR2/$STARTUPLINK
  rm -f $STARTUPDIR3/$STARTUPLINK
  rm -f $STARTUPDIR4/$STARTUPLINK
  rm -f $SHUTDOWNDIR1/$SHUTDOWNLINK
  rm -f $SHUTDOWNDIR2/$SHUTDOWNLINK
  rm -f $STARTUPSCRIPTDIR/$STARTUPSCRIPTNAME
  rm -f $ROOTDIR/$PROGDIR/$INSTALLERNAME

  # Check if this is a symbolic link.
  if ( [ -h $SHELLLINKDIR/$SHELLNAME ] ); then 
    rm -f $SHELLLINKDIR/$SHELLNAME 
  fi

  # Remove program directories  
  rmdir $ROOTDIR/$PROGDIR/$BINDIR 2> /dev/null
  rmdir $ROOTDIR/$PROGDIR 2> /dev/null

  echo "***  Uninstallation successful!"
}


###############################################################################
#
# Script entry point

if ( [ `whoami` != "root" ] ); then
  echo "Quest Software (Provision Networks) USB Server installation script"
  echo "This script is intended to be run under root account!"
  exit 255
fi

case "$1" in
  'install')
    usbsrv_install
    ;;
  'uninstall')
    usbsrv_uninstall
    ;;
  *)
    echo "Quest Software (Provision Networks) USB Server installation script"
    echo ""
    echo "Usage: installer.sh install"
    echo "       installer.sh uninstall"
esac


