
/******************************************************************************
 *
 *  Quest Software, Inc.
 * ALL RIGHTS RESERVED.
 *
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 * 
 *  *  Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.  
 * 
 *  *  Redistributions in binary form must reproduce the above copyright notice, 
 *     this list of conditions and the following disclaimer in the documentation 
 *     and/or other materials provided with the distribution.  
 * 
 *  *  Neither the name of Quest Software, Inc. nor the names of its contributors 
 *     may be used to endorse or promote products derived from this software 
 *     without specific prior written permission. 
 * 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ON AN "AS IS" 
 * AND UNSUPPORTED BASIS, WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES OF ANY KIND, 
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, NON-
 * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE COPYRIGHT 
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ************************************************************************************/


#include "dialogpages.h"


//----------------------------------------------------------------------------
//		Connection dialog page
//----------------------------------------------------------------------------
ConnectionDialogPage::ConnectionDialogPage (ConnectionDialog *parent)
	: QWidget(parent),
	  titleLayout(NULL),
	  parentDialog(parent)
{
}

//----------------------------------------------------------------------------
void ConnectionDialogPage::setTitle(const QString& title,
									const QString& subTitle)
{
	if (titleLayout == NULL)
	{
    	QFont font;
    	font.setBold(true);
    	font.setPointSize(this->font().pointSize()+1);
    	titleText = new QLabel(title);
    	titleText->setFont(font);
    	titleText->setIndent(20);
		subTitleText = new QLabel(subTitle);
		subTitleText->setIndent(40);
	    subTitleText->setWordWrap(true);
    	titleFrame = new QFrame;
    	titleFrame->setFrameStyle(QFrame::HLine | QFrame::Sunken);
    	titleFrame->setFixedHeight(2);
    	titleLayout = new QVBoxLayout;
    	titleLayout->addWidget(titleText);
    	titleLayout->addWidget(subTitleText);
    	titleLayout->addWidget(titleFrame);
	}
	else
	{
		titleText->setText(title);
		if (subTitle != "")
		{
			subTitleText->setText(subTitle);
		}
	}
}

//----------------------------------------------------------------------------
void ConnectionDialogPage::registerField(QString fieldName, QWidget *fieldWidget)
{
	fields.insert(fieldName, fieldWidget);
	if (fieldName.endsWith("*"))
	{
    	QLineEdit *le = qobject_cast<QLineEdit *>(fieldWidget);
    	if (le != NULL)
    	{
    	    connect (le, SIGNAL(textChanged(const QString&)),
        			 this, SLOT(setCompleted(const QString&)));
        }
    }
}

//----------------------------------------------------------------------------
void ConnectionDialogPage::setCompleted(const QString& text)
{
	if (text.isEmpty() || text.trimmed().isEmpty())
	{
		parentDialog->setNextEnabled(false);
	}
	else
	{
    	if (checkComplete() == 0)
    	{
    		parentDialog->setNextEnabled(true);
    	}
	}
}

//----------------------------------------------------------------------------
bool ConnectionDialogPage::checkComplete()
{
	QMap < QString, QWidget* >::const_iterator i = fields.constBegin();
	while (i != fields.constEnd())
	{
        QLineEdit *le =  qobject_cast<QLineEdit *>(i.value());
        if (le != NULL && le->isEnabled())
		{
		    QString t = le->text();
		    int p = 0;
            if (le->validator())
            {
                if (le->validator()->validate(t, p) != QValidator::Acceptable)
                    return 1;
            }
            else if (i.key().endsWith("*") && le->text() == "")
				return 1;
        }
		++i;
	}
	return 0;
}

//----------------------------------------------------------------------------
QMap < QString, QWidget * >::const_iterator ConnectionDialogPage::firstField()
{
	if (!fields.isEmpty())
	{
		return fields.constBegin();
	}
	return (QMap < QString, QWidget * >::const_iterator)NULL;
}

//----------------------------------------------------------------------------
QMap < QString, QWidget * >::const_iterator ConnectionDialogPage::endField()
{
	if (!fields.isEmpty())
	{
		return fields.constEnd();
	}
	return (QMap < QString, QWidget * >::const_iterator)NULL;
}

//----------------------------------------------------------------------------
void ConnectionDialogPage::setField(const QString& name, const QString& value)
{
	QMap < QString, QWidget * >::iterator i = fields.find(name);
	if (i != fields.end())
	{
		QLineEdit *le;
		QCheckBox *chb;
		QGroupBox *gb;
		QRadioButton *rb;
		QComboBox *cmb;
		QSpinBox *sb;
		if ((le = qobject_cast <QLineEdit *> (i.value())) != NULL)
		{
			le->setText(value);
		}
		else if ((chb = qobject_cast <QCheckBox *> (i.value())) != NULL)
		{
			bool ok;
			bool bval = value.toInt(&ok);
			if (ok)
			{
				chb->setChecked(bval);
			}
		}
		else if ((gb = qobject_cast <QGroupBox *> (i.value())) != NULL)
		{
			bool ok;
			bool bval = value.toInt(&ok);
			if (ok)
			{
				gb->setChecked(bval);
			}
        }
		else if ((rb = qobject_cast <QRadioButton *> (i.value())) != NULL)
		{
			bool ok;
			bool bval = value.toInt(&ok);
			if (ok)
			{
				rb->setChecked(bval);
			}
		}
		else if ((cmb = qobject_cast <QComboBox *> (i.value())) != NULL)
		{
			// search for an item with (item.text == value)
			for (int i = 0; i < cmb->count(); ++i)
			{
			    if (cmb->itemText(i) == value)
			    {
			        cmb->setCurrentIndex(i);
			        return;
			    }
			}
            // a scratch to set custom video window size
            if (name == "windowSize")
            {
                cmb->setCurrentIndex(cmb->findText("custom"));
            }
            else
            {
                cmb->setCurrentIndex(0);
            }
		}
		else if ((sb = qobject_cast <QSpinBox *> (i.value())) != NULL)
		{
            bool ok;
            int ival = value.toInt(&ok);
            if (ok)
            {
                sb->setValue(ival);
            }
        }
	}
}
