
/******************************************************************************
 *
 *  Quest Software, Inc.
 * ALL RIGHTS RESERVED.
 *
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 * 
 *  *  Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.  
 * 
 *  *  Redistributions in binary form must reproduce the above copyright notice, 
 *     this list of conditions and the following disclaimer in the documentation 
 *     and/or other materials provided with the distribution.  
 * 
 *  *  Neither the name of Quest Software, Inc. nor the names of its contributors 
 *     may be used to endorse or promote products derived from this software 
 *     without specific prior written permission. 
 * 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ON AN "AS IS" 
 * AND UNSUPPORTED BASIS, WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES OF ANY KIND, 
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, NON-
 * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE COPYRIGHT 
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ************************************************************************************/


#include <QtGui>
#include "conndialog.h"
#include "dialogpages.h"
#include "mainwindow.h"
#include "../common/crypto.h"

//----------------------------------------------------------------------------
//		Connection Dialog
//----------------------------------------------------------------------------
ConnectionDialog::ConnectionDialog(QWidget *parent)
	: QDialog(parent),
	  pntsc(NULL)
{
    if (parent)
        parentWindow = qobject_cast <ConnectionManagerMainWindow *>(parent);

	titleLabel = new QLabel;
	titleLabel->setMinimumHeight(0);
	titleLabel->setMinimumWidth(0);
	titleLabel->setMaximumHeight(80);
	titleLabel->setMaximumWidth(720);
	titleLabel->setFixedHeight(34);
	titleLabel->setFixedWidth(720);

	pagesStackedWidget = new QStackedWidget;
	pagesStackedWidget->addWidget(new SessionNamePage(this));
	pagesStackedWidget->addWidget(new CredentialsPage(this));
	pagesStackedWidget->addWidget(new ConnectionPage(this));
	pagesStackedWidget->addWidget(new ApplicationPage(this));
	pagesStackedWidget->addWidget(new DisplayOptionsPage(this));
	pagesStackedWidget->addWidget(new ClientOptionsPage(this));
	pagesStackedWidget->addWidget(new DrivesMappingOptionsPage(this));
	pagesStackedWidget->addWidget(new LPTPortsMappingOptionsPage(this));
	pagesStackedWidget->addWidget(new COMPortsMappingOptionsPage(this));
	pagesStackedWidget->addWidget(new RDPPrintersOptionsPage(this));
	pagesStackedWidget->addWidget(new SmartCardsOptionsPage(this));
	pagesStackedWidget->addWidget(new ExperienceOptionsPage(this));
	pagesStackedWidget->setCurrentIndex(0);
	leftList = new QListWidget;
	leftList->setFixedWidth(150);
	leftList->addItem("Session name");
	leftList->addItem("Credentials");
	leftList->addItem("Connection options");
	leftList->addItem("Start application");
	leftList->addItem("Display settings");
	leftList->addItem("Client general options");
	leftList->addItem("Client drives");
	leftList->addItem("Client LPT ports");
	leftList->addItem("Client COM ports");
	leftList->addItem("RDP printers");
	leftList->addItem("Smart cards");
	leftList->addItem("Experience options");
	leftList->setCurrentRow(0);
	bottomLine = new QFrame;
	bottomLine->setFrameStyle(QFrame::HLine | QFrame::Sunken);
	bottomLine->setFixedHeight(2);

    bottomButtonBox = new QDialogButtonBox(Qt::Horizontal);
    bottomButtonBox->setContentsMargins(10,0,10,8);

    QHBoxLayout *hLayout = new QHBoxLayout;
    hLayout->addWidget(leftList);
    hLayout->addWidget(pagesStackedWidget);
    hLayout->setContentsMargins(5, 0, 5, 0);
    QVBoxLayout *vLayout = new QVBoxLayout;
    vLayout->setSpacing(4);
    vLayout->addWidget(titleLabel);
    vLayout->addLayout(hLayout);
    vLayout->addWidget(bottomLine);
    vLayout->addWidget(bottomButtonBox);
    vLayout->setContentsMargins(10,8,10,10);
    setLayout(vLayout);

    setContentsMargins(-13,-13,-13,-10);
	setMinimumSize(QSize(700,450));
	setMaximumSize(QSize(700,450));
}

//----------------------------------------------------------------------------
void ConnectionDialog::saveParameters(const QString& filename, const QStringList& paramsList)
{
    QFile file(filename);
    if (!file.open(QIODevice::WriteOnly | QIODevice::Text | QIODevice::Truncate))
    {
        QMessageBox::critical(this, "Connection Editor", "Can't open file \"" +  filename + "\"");
        QDialog::reject();
    }
    QTextStream stream(&file);

    QString parameter;
    for (int i = 0; i < paramsList.size(); i++)
    {
        parameter = paramsList.at(i);
        // encrypt password and application name if it's necessary
        if (parameter.startsWith("password:s:"))
        {
    	    if ( parameter != "password:s:")
    	    {
        	unsigned char buffer[256];
        	char szPassword[257*2];
        	::Encode256(buffer, parameter.remove("password:s:").toLatin1().data());
        	::Encrypt(buffer, sizeof (buffer));
        	::BinToHex(buffer, sizeof (buffer), szPassword, sizeof(szPassword));
        	parameter = szPassword;
        	parameter.prepend("password:s:");
    	    }
        }
        else if (parameter.startsWith("alternate shell:s:"))
        {
    	    if (parameter != "alternate shell:s:")
    	    {
    		unsigned char buffer[1024];
    		char szShell[1025*2];
    		::Encode1024(buffer, parameter.remove("alternate shell:s:").toLatin1().data());
    		::Encrypt(buffer, sizeof (buffer));
    		::BinToHex(buffer, sizeof (buffer), szShell, sizeof(szShell));
    		parameter = szShell;
    		parameter.prepend("alternate shell:s:");
    	    }
        }

    	stream << parameter << endl;

    }
    file.close();
}


//----------------------------------------------------------------------------
void ConnectionDialog::getParameters(const QString& filename, QStringList& parameters)
{
	QFile file(filename);
    if (!file.open(QIODevice::ReadOnly | QIODevice::Text))
    {
        QMessageBox::critical(this, "Connection Editor", "Can't open file \"" +  filename + "\"");
        return;
    }
    parameters.clear();
    QTextStream stream(&file);
    QString line, name, val;
    while (!stream.atEnd()) {
        name = line = stream.readLine();
        if (line == "") break;
        int pos = line.indexOf(":");
        name.truncate(pos+3);
        val = line.remove(0, pos + 3);
        if (val != "")
        {
            if (name == "password:s:" && val.length() == 512)
            {
            	unsigned char buffer[258];
            	char passw[64];
            	::HexToBin(buffer, sizeof(buffer), val.toLatin1().data(), val.length());
            	::Decrypt(buffer, sizeof(buffer));
            	::Decode256(buffer, passw);
            	line = passw;
            	line.prepend(name);
            }
            else if (name == "alternate shell:s:" && val.length() == 2048)
            {
            	unsigned char buffer[1026];
            	char shell[256];
            	::HexToBin(buffer, sizeof(buffer), val.toLatin1().data(), val.length());
            	::Decrypt(buffer, sizeof(buffer));
            	::Decode1024(buffer, shell);
            	line = shell;
            	line.prepend(name);
            }
            else
            {
        	line = name + val;
            }
        }
        else
        {
    	    line = name;
        }
        parameters.append(line);
    }
}

//----------------------------------------------------------------------------
QString ConnectionDialog::findField(const QString &name)
{
	QString res("");
	QMap < QString, QString >::const_iterator it = parameters.find(name);
	if (it != parameters.end())
	{
		res = it.value();
	}
	return res;
}

//----------------------------------------------------------------------------
QStringList ConnectionDialog::refreshAppsList(int mask)
{
    QStringList apps;
    QString broker, brokerPort, brokerProtocol;
    QString type;
    QString wait, retry;
    if (pntsc != NULL)
        return apps;
    if (mask == 1) type = "-La";
    else if (mask == 2) type = "-Ld";
    else type = "-L";

    for (int i = 0; i < pagesStackedWidget->count(); i++)
    {
    	ConnectionDialogPage *page =
    		qobject_cast <ConnectionDialogPage *> (pagesStackedWidget->widget(i));
	    if (page != NULL)
	    {
		    QMap < QString, QWidget* >::const_iterator it = page->firstField();
		    QMap < QString, QWidget* >::const_iterator end_it = page->endField();
		    while (it != end_it)
		    {
		    	QString name = it.key();
           		QLineEdit *le;
           		QComboBox *cb;
           		QSpinBox *sb;
		    	if (name == "username*" && (le = qobject_cast <QLineEdit *> (it.value())) != NULL)
		    	{
            			username = le->text();
		    	}
		    	else if (name == "password*" && (le = qobject_cast <QLineEdit *> (it.value())) != NULL)
		    	{
            			password = le->text();
		    	}
		    	else if (name == "domain*" && (le = qobject_cast <QLineEdit *> (it.value())) != NULL)
		    	{
            			domain = le->text();
		    	}
		    	else if (name == "llmservers*" && (le = qobject_cast <QLineEdit *> (it.value())) != NULL)
		    	{
            			broker = le->text();
		    	}
		    	else if (name == "llmtcpport*" && (le = qobject_cast <QLineEdit *> (it.value())) != NULL)
		    	{
            			brokerPort = le->text();
		    	}
		    	else if (name == "llmprotocol" && (cb = qobject_cast <QComboBox *> (it.value())) != NULL)
		    	{
            			brokerProtocol = cb->currentText().toLower() + "://";
		    	}
		    	else if (name == "wait" && (sb = qobject_cast <QSpinBox *> (it.value())) != NULL)
		    	{
            			wait.setNum(sb->value());
		    	}
		    	else if (name == "retry" && (sb = qobject_cast <QSpinBox *> (it.value())) != NULL)
		    	{
            			retry.setNum(sb->value());
		    	}
		    	++it;
		    }
    	}
    }
    if (broker == "")
    {
	    return apps;
    }
    if (username == "" || password == "" || domain == "")
    {
		CredentialsForm form(this);
		if (form.exec() == QDialog::Rejected)
		{
			return apps;
		}
    }
    if (brokerPort != "")
    {
        broker.append(QString(":") + brokerPort);
    }
    broker.prepend(brokerProtocol);
    pntsc = new QProcess;
    connect (pntsc, SIGNAL(readyReadStandardError()),
    		 this, SLOT(getPntscErrors()));
    pntsc->start("pntsc", QStringList() << broker << "-u" << username
					<< "-p" << password << "-d" << domain
					<< "-w" << wait << "-n" << retry
					<< type);
    QString errstr;
    if (!pntsc->waitForStarted(1000))
    {
	    QMessageBox::critical(this, "PNTSC", "Error starting PNTSC");
    }
    else
    {
        while (pntsc->state() == QProcess::Running)
        {
            QApplication::processEvents();
        }
    }
    QByteArray data = pntsc->readAllStandardOutput();
    QStringList list = QString(data).split("\n", QString::SkipEmptyParts);
    foreach (QString line, list)
    {
        if (line.startsWith("Name="))
        {
            // the application entry is given in the following format:
            // Name=%s[,FolderName=%s][,LargeIcon=%s][,SmallIcon=%s]
            line = line.remove("Name=");
            int pos = line.indexOf(",");
            if (pos != -1)
            {
                // remove the tail with the unnecessary data
                line = line.remove(pos, line.length() - pos);
            }
            apps.append(line);
        }
    }
    delete pntsc;
    pntsc = NULL;
	return apps;
}

//----------------------------------------------------------------------------
void ConnectionDialog::getPntscErrors()
{
	QByteArray data = pntsc->readAllStandardError();

	// parse received error string
	QStringList list = QString(data).split("],");
	QString code = list.at(0);
	QString module = list.at(1);
	QString inf = list.at(2);
    code.replace("[ERR","");
    module.replace("[","");

    bool ok;
    int errcode = code.toInt(&ok, 16);
    QString errcodestr = getPntscErrorString(errcode);

    QMessageBox::critical(this, "Getting an applications list",
        "[Error " + code + "]: " + errcodestr +
        "\n[Module]: " + module +
        "\n[Description]: " + inf);
}

//----------------------------------------------------------------------------
void ConnectionDialog::setCredentials(const QString& u, const QString& p,
									  const QString& d)
{
	username = u;
	password = p;
	domain = d;
}

//----------------------------------------------------------------------------
void ConnectionDialog::addDomain(const QString& domain)
{
    if (parentWindow)
        parentWindow->addDomain(domain);
}

//----------------------------------------------------------------------------
QStringList ConnectionDialog::domainList()
{
   	if (parentWindow)
   	    return parentWindow->domainList();
    return QStringList();
}

//----------------------------------------------------------------------------
void ConnectionDialog::accept()
{
    for (int i = 0; i < pagesStackedWidget->count(); i++)
    {
    	ConnectionDialogPage *page =
    		qobject_cast <ConnectionDialogPage *> (pagesStackedWidget->widget(i));
	    if (page != NULL)
	    {
		    QMap < QString, QWidget* >::const_iterator it = page->firstField();
		    QMap < QString, QWidget* >::const_iterator end_it = page->endField();
		    while (it != end_it)
		    {
		    	QString name = it.key();
		    	if (name.endsWith("*"))
		    	{
		    		name.truncate(name.length()-1);
		    	}
		    	QString value("");
        		QLineEdit *le;
        		QCheckBox *chb;
        		QGroupBox *gb;
        		QRadioButton *rb;
        		QComboBox *cmb;
        		QSpinBox *sb;
        		if ((le = qobject_cast <QLineEdit *> (it.value())) != NULL)
        		{
        			if (le->isEnabled() || name == "sessionname")
        			{
        			    value = le->text();
        			}
        			value.prepend("s:");
        		}
        		else if ((chb = qobject_cast <QCheckBox *> (it.value())) != NULL)
        		{
       				value.setNum(chb->isChecked());
       				value.prepend("i:");
        		}
        		else if ((gb = qobject_cast <QGroupBox *> (it.value())) != NULL)
        		{
       				value.setNum(gb->isChecked());
       				value.prepend("i:");
                }
        		else if ((rb = qobject_cast <QRadioButton *> (it.value())) != NULL)
        		{
       				value.setNum(rb->isChecked());
       				value.prepend("i:");
        		}
        		else if ((cmb = qobject_cast <QComboBox *> (it.value())) != NULL)
        		{
       				if (cmb->currentIndex() != -1)
       				{
       				    if (cmb->itemText(cmb->currentIndex()) == "custom" && name == "windowSize")
       				    {
                		    QMap < QString, QWidget* >::const_iterator it = page->firstField();
		                    QMap < QString, QWidget* >::const_iterator end_it = page->endField();
		                    QString customHeight = "", customWidth = "";
		                    while (it != end_it)
		                    {
                		    	QString name = it.key();
		                        if (name == "customWindowHeight")
		                        {
                            		QLineEdit *le = qobject_cast <QLineEdit *> (it.value());
		                            customHeight = le->text();
		                        }
		                        if (name == "customWindowWidth")
		                        {
                            		QLineEdit *le = qobject_cast <QLineEdit *> (it.value());
		                            customWidth = le->text();
		                        }
		                        ++it;
		                    }
		                    if (customHeight == "" || customWidth == "")
		                    {
		                        customHeight = "600";
		                        customWidth = "800";
                            }
		                    value = customWidth + "x" + customHeight;
       				    }
       				    else
       				    {
       					    value = cmb->itemText(cmb->currentIndex());
                        }
       				}
       				value.prepend("s:");
        		}
        		else if ((sb = qobject_cast <QSpinBox *> (it.value())) != NULL)
        		{
       				value.setNum(sb->value());
       				value.prepend("i:");
        		}
		    	parameters[name] = value;
		    	++it;
		    }
		}
    }
    parameters.remove("saveStartupApp");
    parameters.remove("saveCredentials");

    QString filename(findField("sessionname").remove(0, 2) + ".rdp");
    parameters.remove("sessionname");

    QStringList paramsStringList;
   	QMap < QString, QString >::iterator it;

    for (it = parameters.begin(); it != parameters.end(); it++)
    {
        paramsStringList.append(it.key() + ":" + it.value());
    }

    saveParameters(filename, paramsStringList);

    QDialog::accept();
}

//----------------------------------------------------------------------------
//		Connection Wizard
//----------------------------------------------------------------------------
ConnectionWizard::ConnectionWizard(QWidget *parent)
    : ConnectionDialog(parent)
{
	titleLabel->setPixmap(QPixmap(":/images/label_new.png"));
	pagesStackedWidget->addWidget(new StartupPage(this));
	pagesStackedWidget->setCurrentIndex(0);
	leftList->setEnabled(false);
	leftList->addItem("Finish");
	leftList->setCurrentRow(0);
	QPalette palette;
	palette.setColor(QPalette::Disabled, QPalette::Text, QColor(Qt::black));
	palette.setColor(QPalette::Disabled, QPalette::HighlightedText, QColor(Qt::black));
	leftList->setPalette(palette);
	MyPushButtonCreator creator;
    backButton = creator.createButton("< &Back");
    nextButton = creator.createButton("&Next >");
    cancelButton = creator.createButton("Cancel");
    backButton->setEnabled(false);
    nextButton->setEnabled(false);
    bottomButtonBox->addButton(backButton, QDialogButtonBox::ActionRole);
    bottomButtonBox->addButton(nextButton, QDialogButtonBox::ActionRole);
    bottomButtonBox->addButton(cancelButton, QDialogButtonBox::RejectRole);
    setTabOrder(pagesStackedWidget, nextButton);
    setTabOrder(nextButton, backButton);
    connect(backButton, SIGNAL(clicked()), this, SLOT(back()));
    connect(nextButton, SIGNAL(clicked()), this, SLOT(next()));
    connect(cancelButton, SIGNAL(clicked()), this, SLOT(close()));
    connect(pagesStackedWidget, SIGNAL(currentChanged(int)), this, SLOT(checkComplete(int)));
    setWindowTitle(tr("Remote Desktop Connection Wizard"));
}

//----------------------------------------------------------------------------
void ConnectionWizard::checkComplete(int pageIndex)
{
   	ConnectionDialogPage *page =
   		qobject_cast <ConnectionDialogPage *> (pagesStackedWidget->widget(pageIndex));
    if (page != NULL)
    {
		if (page->checkComplete() != 0)
		{
			nextButton->setEnabled(false);
		}
		else
		{
			nextButton->setEnabled(true);
		}
	}
}

//----------------------------------------------------------------------------
void ConnectionWizard::back()
{
	if (pagesStackedWidget->currentIndex() == pagesStackedWidget->count()-1)
	{
		nextButton->setText("&Next >");
	    disconnect(nextButton, SIGNAL(clicked()), this, SLOT(accept()));
	    connect(nextButton, SIGNAL(clicked()), this, SLOT(next()));
	}
	if (pagesStackedWidget->currentIndex() == 1)
	{
	    backButton->setEnabled(false);
	}
	leftList->setCurrentRow(leftList->currentRow()-1);
	pagesStackedWidget->setCurrentIndex(pagesStackedWidget->currentIndex()-1);
}

//----------------------------------------------------------------------------
void ConnectionWizard::next()
{
	if (pagesStackedWidget->currentIndex() == pagesStackedWidget->count() - 2)
	{
		nextButton->setText("&Finish");
	    disconnect(nextButton, SIGNAL(clicked()), this, SLOT(next()));
	    connect(nextButton, SIGNAL(clicked()), this, SLOT(accept()));
	}
	if (pagesStackedWidget->currentIndex() == 0)
	{
		backButton->setEnabled(true);
	}
	leftList->setCurrentRow(leftList->currentRow()+1);
	pagesStackedWidget->setCurrentIndex(pagesStackedWidget->currentIndex()+1);
}

//----------------------------------------------------------------------------
void ConnectionWizard::setNextEnabled(bool value)
{
    nextButton->setEnabled(value);
}

//----------------------------------------------------------------------------
void ConnectionWizard::accept()
{
    ConnectionDialog::accept();
}

//----------------------------------------------------------------------------
//		Connection editor
//----------------------------------------------------------------------------
ConnectionEditor::ConnectionEditor(QString& pitFileName, QWidget *parent)
    : ConnectionDialog(parent)
{
	titleLabel->setPixmap(QPixmap(":/images/label_edit.png"));
	MyPushButtonCreator creator;
    okButton = creator.createButton("OK");
    bottomButtonBox->addButton(okButton, QDialogButtonBox::YesRole);
    connect(okButton, SIGNAL(clicked()), this, SLOT(accept()));
    connect(leftList, SIGNAL(currentRowChanged(int)),
            pagesStackedWidget, SLOT(setCurrentIndex(int)));
    setWindowTitle(QString("Virtual Access Connection Editor - %1").arg(pitFileName));

    QStringList paramsStringList;
    getParameters(pitFileName, paramsStringList);
    for (int i = 0; i < paramsStringList.size(); i++)
    {
        QString parameter = paramsStringList.at(i);
        QString name(parameter);
        QString val(parameter);
        name.truncate(parameter.indexOf(":"));
        val.remove(0, parameter.indexOf(":")+3);
        parameters.insert(name, val);
    }
    for (int i = 0; i < pagesStackedWidget->count(); i++)
    {
    	ConnectionDialogPage *page =
    		qobject_cast <ConnectionDialogPage *> (pagesStackedWidget->widget(i));
	    if (page != NULL)
	    {
		    QMap < QString, QWidget* >::const_iterator it = page->firstField();
		    QMap < QString, QWidget* >::const_iterator end_it = page->endField();
		    while (it != end_it)
		    {
		    	QString name = it.key();
		    	if (name.endsWith("*"))
		    	{
		    		name.truncate(name.length()-1);
		    	}
		    	QString value;
		    	if (name == "sessionname")
		    	{
		    		value = QString(pitFileName).remove(".rdp");
		    	}
		    	else
		    	{
		    		value = findField(name);
		    	}
		    	if (value != "")
		    	{
		    		page->setField(it.key(), value);
		    	}
		    	it++;
		    }
		}
    }
}

//----------------------------------------------------------------------------
void ConnectionEditor::accept()
{
	ConnectionDialog::accept();
}

//----------------------------------------------------------------------------
void ConnectionEditor::setNextEnabled(bool value)
{
    okButton->setEnabled(value);
}


//    QMessageBox::information(this, "setField", QString("%1 %2").arg(name).arg(value));
