
/******************************************************************************
 *
 *  Quest Software, Inc.
 * ALL RIGHTS RESERVED.
 *
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 * 
 *  *  Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.  
 * 
 *  *  Redistributions in binary form must reproduce the above copyright notice, 
 *     this list of conditions and the following disclaimer in the documentation 
 *     and/or other materials provided with the distribution.  
 * 
 *  *  Neither the name of Quest Software, Inc. nor the names of its contributors 
 *     may be used to endorse or promote products derived from this software 
 *     without specific prior written permission. 
 * 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ON AN "AS IS" 
 * AND UNSUPPORTED BASIS, WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES OF ANY KIND, 
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, NON-
 * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE COPYRIGHT 
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ************************************************************************************/


#include "pnsslcli.h"
#include <syslog.h>

#include <boost/bind.hpp>
#include <assert.h>

// ------------------------------------------------------------------------------
// session member functions realization
// ------------------------------------------------------------------------------
session::session (asio::io_service& io_service,
                  asio::ssl::context& context)
    : socket_ (io_service),
      ssl_socket_ (io_service, context),
      timer_ (io_service)
{
}

// ------------------------------------------------------------------------------
tcp::socket& session::socket ()
{
    return socket_;
}

// ------------------------------------------------------------------------------
ssl_socket& session::sslsocket ()
{
    return ssl_socket_;
}

// ------------------------------------------------------------------------------
void session::start (const char *rdphost, const char *username)
{
    char buf[128];
    memset (buf, '\0', 128);

    int sendbytes = sprintf (buf, "CONNECT: %s\r\nCLIENT: %s\r\n\r\n", rdphost, username);
    syslog (LOG_NOTICE, buf);
    ssl_socket_.write_some (asio::buffer (buf, sendbytes));
    
    sleep (2);  // give VPN server time to prepare
    
    ssl_socket_.async_read_some (asio::buffer (data_in_),
        make_custom_alloc_handler (allocator_in_,
            boost::bind (&session::handle_server_read,
               shared_from_this (),
               asio::placeholders::error,
               asio::placeholders::bytes_transferred)));
    socket_.async_read_some (asio::buffer (data_out_),
        make_custom_alloc_handler (allocator_out_,
            boost::bind (&session::handle_read,
                   shared_from_this (),
                   asio::placeholders::error,
                   asio::placeholders::bytes_transferred)));
}

// ------------------------------------------------------------------------------
void session::on_timer (const asio::error_code& e)
{
    if (!e)
    {
        syslog (LOG_NOTICE, "write operation timeout");
        ssl_socket_.lowest_layer().close();
    }
}

// ------------------------------------------------------------------------------
void session::handle_read (const asio::error_code & error, size_t bytes_transferred)
{
    if (!error)
    {
        timer_.expires_from_now (boost::posix_time::seconds(10));
        timer_.async_wait (boost::bind (&session::on_timer,
                                        shared_from_this (),
                                        asio::placeholders::error));
        asio::async_write (ssl_socket_, 
                asio::buffer (data_out_, bytes_transferred),
                make_custom_alloc_handler (allocator_out_,
                    boost::bind (&session::handle_server_write,
                        shared_from_this (),
                        asio::placeholders::error)));
    }
    else
    {
      throw error;
    }
}

// ------------------------------------------------------------------------------
void session::handle_server_read (const asio::error_code & error, size_t bytes_transferred)
{
    if (!error)
    {
        asio::async_write (socket_,
                asio::buffer (data_in_, bytes_transferred),
                make_custom_alloc_handler (allocator_in_,
                    boost::bind (&session::handle_write,
                        shared_from_this (),
                        asio::placeholders::error)));
    }
    else
    {
      syslog (LOG_NOTICE, "server_read error: %s", error.message().c_str());
      throw error;
    }
}

// ------------------------------------------------------------------------------
void session::handle_write (const asio::error_code & error)
{
    if (!error)
    {
        ssl_socket_.async_read_some (asio::buffer (data_in_),
            make_custom_alloc_handler (allocator_in_,
                boost::bind (&session::handle_server_read,
                    shared_from_this (),
                    asio::placeholders::error,
                    asio::placeholders::bytes_transferred)));
    }
    else
    {
      throw error;
    }
}

// ------------------------------------------------------------------------------
void session::handle_server_write (const asio::error_code & error)
{
    timer_.cancel();
    if (!error)
    {
        socket_.async_read_some (asio::buffer (data_out_),
                make_custom_alloc_handler (allocator_out_,
                    boost::bind (&session::handle_read,
                        shared_from_this (),
                        asio::placeholders::error,
                        asio::placeholders::bytes_transferred)));
    }
    else
    {
      throw error;
    }
}
