
/******************************************************************************
 *
 *  Quest Software, Inc.
 * ALL RIGHTS RESERVED.
 *
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 * 
 *  *  Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.  
 * 
 *  *  Redistributions in binary form must reproduce the above copyright notice, 
 *     this list of conditions and the following disclaimer in the documentation 
 *     and/or other materials provided with the distribution.  
 * 
 *  *  Neither the name of Quest Software, Inc. nor the names of its contributors 
 *     may be used to endorse or promote products derived from this software 
 *     without specific prior written permission. 
 * 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ON AN "AS IS" 
 * AND UNSUPPORTED BASIS, WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES OF ANY KIND, 
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, NON-
 * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE COPYRIGHT 
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ************************************************************************************/



#include "../common/common.h"

#include "rdesktop.h"
#include "qplayer.h"
#include "qaudio_driver.h"
#include "wave_buffer.h"

/*--------------------------------------------------------------------------------*/
int g_qplayer_buffering = 0;


void * playback_thread_process ( void * _context );

typedef struct
{
	pthread_t handle;
	int		  terminate;
	audio_player * player;
	uint32 	     buffering;
} thread_context;

static int bytes_from_time ( int _buffering )
{
    return  _buffering * 22.05f * 2 * sizeof(short);
}

thread_context * playback_thread_create ( audio_player * _player )
{
	thread_context * context = NULL;

	context = (thread_context*)xmalloc ( sizeof ( thread_context ));
	if ( context == NULL )
		return NULL;

	context->player = _player;
	context->terminate = 0;
	context->buffering = bytes_from_time( g_qplayer_buffering );

	pthread_create(&context->handle, NULL, &playback_thread_process, context );

	return context;
}

void playback_thread_destroy ( thread_context * _context )
{
	xfree(_context);
}

/*---------------------------------------------------------------------------------*/

void * playback_thread_process ( void * _context )
{
	thread_context * context = (thread_context*) _context;

        syslog( LOG_NOTICE, "Starting a playback thread" );

	while (!context->terminate)
	{
		sync_wave_buffer * wb = (sync_wave_buffer*) context->player->wave_buffer;

		if ( wave_buffer_get_writed_bytes( wb ) > context->buffering )
		{
			uint8 data_buffer [ 1024 ]; /* 1 kb chunk size*/
			int read = 0, total = 0;

			do
			{
				read = wave_buffer_read(wb , data_buffer, sizeof( data_buffer) );
				if ( read )
				{
					total += read;
					audio_player * player = context->player;
					player->driver->waveOutWrite ( player->handle , data_buffer , read );
				}
			} while ( read );
			context->buffering = 0;
			if ( total < bytes_from_time (g_qplayer_buffering ) / 4)
			{
			    context->buffering = bytes_from_time(g_qplayer_buffering);
			}
		}
		else
		{
			int i = 0;
			for ( i = 0; i < 100; ++ i)
			{
			    usleep( 10 * 1000 ); /* reduce CPU utilization */
			    if ( context->terminate )
				break;
			}
		}
	}

        syslog( LOG_NOTICE, "Ending a playback thread" );

	return NULL;
}

extern qaudio_waveout_driver waveout_driver;

static int is_correct_audio_driver ( )
{
    if (
       ( waveout_driver.waveOutOpen == NULL )  ||
       ( waveout_driver.waveOutClose == NULL ) ||
       ( waveout_driver.waveOutReset == NULL ) ||
       ( waveout_driver.waveOutWrite == NULL ) )
    return 0;
    
    if ( ( waveout_driver.driver < 0 ) || ( waveout_driver.driver > 6 ) )
        return 0;
    
    return 1;
}

audio_player * create_player (  WAVEFORMATEX * _format )
{        
    audio_player * player = NULL;

    if ( !is_correct_audio_driver() )
        return NULL;
    
    player = (audio_player*) xmalloc ( sizeof ( audio_player ) );
    
    if ( player == NULL )
        return NULL;

    syslog( LOG_NOTICE, "An audio player (ref: %p) was created.", player );
        
    player->driver = &waveout_driver;
    
    _format->wBitsPerSample = (_format->nBlockAlign / _format->nChannels) * 8;

    waveout_driver.waveOutOpen( &player->handle, _format );
    
    if ( player->handle == NULL )
    {
        xfree( player );
        return NULL;
    }
    
    player->wave_buffer = (void*) wave_buffer_create( MAX_WAVE_BUFFER_SIZE );
    player->context     = (void*) playback_thread_create ( player );
    player->opened = 1;
        
    return player;
}

void destroy_player ( audio_player * _player )
{
	thread_context * context = NULL;
	if ( _player == NULL )
        return;
    
    reset_player( _player );

    context = (thread_context*) _player->context;
    context->terminate = 1;

    usleep( 100 * 1000 ); /* wait for termination */

    playback_thread_destroy( context );

    wave_buffer_destroy( (sync_wave_buffer*) _player->wave_buffer);

    _player->driver->waveOutClose( _player->handle );

    syslog( LOG_NOTICE, "Audio player ref:%p has been released.", _player );

    xfree ( _player );
}

void configure_player ( audio_player * _player , uint32 _pitch, uint32 _volume , uint32 _rate )
{

}


void play_wave ( audio_player * _player , uint8 * _data , uint32 _size )
{
	int written  = 0;
	if ( (_player == NULL ) || (_data == NULL) || (_size == 0) )
        return;

    /*_player->driver->waveOutWrite ( _player->handle , _data , _size );*/

	do
	{
		written += wave_buffer_write((sync_wave_buffer*)_player->wave_buffer,
				_data + written, _size - written);
		if ( written < _size )
		{
			printf("written < _size \n");
			usleep( 5 * 1000); /* 33 ms chunk, reduce CPU utilization */
		}
	}while ( written < _size );
}

void reset_player( audio_player * _player )
{
	wave_buffer_clear( (sync_wave_buffer*) _player->wave_buffer );
	_player->driver->waveOutReset( _player->handle );
}


