
/******************************************************************************
 *
 *  Quest Software, Inc.
 * ALL RIGHTS RESERVED.
 *
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 * 
 *  *  Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.  
 * 
 *  *  Redistributions in binary form must reproduce the above copyright notice, 
 *     this list of conditions and the following disclaimer in the documentation 
 *     and/or other materials provided with the distribution.  
 * 
 *  *  Neither the name of Quest Software, Inc. nor the names of its contributors 
 *     may be used to endorse or promote products derived from this software 
 *     without specific prior written permission. 
 * 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ON AN "AS IS" 
 * AND UNSUPPORTED BASIS, WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES OF ANY KIND, 
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, NON-
 * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE COPYRIGHT 
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ************************************************************************************/


/*
 * wave_buffer.c
 *
 *  Created on: 05.08.2009
 *      Author: beloborodov
 */
#include "rdesktop.h"

#include "wave_buffer.h"

#include <pthread.h>

sync_wave_buffer * wave_buffer_create (  int _size )
{
	sync_wave_buffer * wave = (sync_wave_buffer*) xmalloc(sizeof( sync_wave_buffer ));
	if (wave == NULL)
		return NULL;

	wave->data = (uint8*) xmalloc ( _size );
	if ( wave->data == NULL)
	{
		xfree( wave );
		return NULL;
	}

	wave->size = _size;
	wave->read_ptr = 0;
	wave->write_ptr = 0;
	wave->write_avail = _size;

	pthread_mutex_init( &wave->sync, NULL );

	return wave;
}

void wave_buffer_destroy( sync_wave_buffer * _wave )
{
	if ( _wave == NULL)
		return;

	pthread_mutex_lock( &_wave->sync );
	xfree ( _wave->data );
	xfree ( _wave );
	pthread_mutex_unlock(&_wave->sync);
	pthread_mutex_destroy( &_wave->sync );
}

void wave_buffer_clear ( sync_wave_buffer * _wave )
{
	pthread_mutex_lock( &_wave->sync);

	_wave->read_ptr = _wave->write_ptr = 0;
	_wave->write_avail = _wave->size;

	pthread_mutex_unlock( &_wave->sync );
}

int wave_buffer_get_size ( sync_wave_buffer * _wave)
{
	int ret = 0;
	if (_wave == NULL )
		return 0;

	pthread_mutex_lock(&_wave->sync);
	ret = _wave->size;
	pthread_mutex_unlock(&_wave->sync);

	return ret;
}

int wave_buffer_read ( sync_wave_buffer * _wave , uint8* _data , int _to_read )
{
	int to_read = 0;

	pthread_mutex_lock(&_wave->sync);

	if ( _wave == NULL || _wave->write_avail == _wave->size || _to_read <= 0 )
	{
		pthread_mutex_unlock(&_wave->sync);
		return 0;
	}



	to_read = ( _wave->size - _wave->write_avail );

	to_read = (to_read < _to_read ) ? to_read : _to_read;

	if ( to_read > _wave->size - _wave->read_ptr)
	{
		int len = _wave->size - _wave->read_ptr;
		memcpy( _data , _wave->data + _wave->read_ptr , len);
		memcpy( _data + len , _wave->data , to_read - len);
	}
	else
	{
		memcpy( _data , _wave->data + _wave->read_ptr , to_read );
	}

	_wave->read_ptr = (_wave->read_ptr + to_read ) % _wave->size;
	_wave->write_avail += to_read;

	pthread_mutex_unlock(&_wave->sync);

	return to_read;
}

int wave_buffer_write ( sync_wave_buffer * _wave , uint8 * _data , int _to_write )
{
	pthread_mutex_lock(&_wave->sync);

	if ( _wave == NULL || _data == NULL || _to_write == 0 )
	{
		pthread_mutex_unlock(&_wave->sync);
		return 0;
	}


	if ( _to_write > _wave->write_avail )
	{
		_to_write = _wave->write_avail;
	}

	if ( _to_write > _wave->size - _wave->write_ptr )
	{
		int len = _wave->size - _wave->write_ptr;
		memcpy( _wave->data + _wave->write_ptr , _data , len);
		memcpy( _wave->data , _data + len , _to_write - len );
	}
	else
	{
		memcpy( _wave->data + _wave->write_ptr , _data, _to_write );
	}

	_wave->write_ptr = (_wave->write_ptr + _to_write ) % _wave->size;
	_wave->write_avail -= _to_write;

	pthread_mutex_unlock(&_wave->sync);

	return _to_write;
}

int wave_buffer_get_writed_bytes ( sync_wave_buffer * _wave)
{
	int ret = 0;
	pthread_mutex_lock( &_wave->sync);

	ret = _wave->size - _wave->write_avail;

	pthread_mutex_unlock( &_wave->sync);

	return ret;
}
